from __future__ import annotations

import math
from typing import List

from gui.impl.opengl import OpenGLGUIFacade
from gui.layer import CharactersLayer
from .OpenGLLayer import OpenGLLayer
from .OpenGLLayerGroup import OpenGLLayerGroup


class OpenGLCharactersLayer(OpenGLLayer, CharactersLayer):

    def __init__(self, gui: OpenGLGUIFacade, layerGroup: OpenGLLayerGroup):
        super(OpenGLCharactersLayer, self).__init__(gui, layerGroup, dynamic=False)
        self.__width = 0  # type: int
        self.__height = 0  # type: int
        self.__depthMax = 0  # type: int
        self.__depthStep = 0  # type: int
        self.__depths = []  # type: List[int]

    def updateDepths(self, minDepth: int, maxDepth: int, index: int, maxIndex: int) -> int:
        self.__depthMax = maxDepth - index
        self.__depthStep = maxIndex
        self.__depths = [self.__depthMax - y * self.__depthStep for y in range(self.__height)]
        self.setFacesDepth(self.__depthMax)
        return self.__depthMax - self.__height * self.__depthStep

    def hasDepth(self, depth: int) -> bool:
        return depth in self.__depths

    def setGridSize(self, width: int, height: int):
        self.__width = width
        self.__height = height

    def getCharacterCount(self) -> int:
        return self.faceCount

    def setCharacterCount(self, count: int):
        self.createFaces(count, self.__depthMax)

    def setCharacterLocation(self, charIndex: int, x: float, y: float):
        self.setFaceTileLocation(charIndex, x, y)
        y = math.ceil(y)
        depth = self.__depthMax - y * self.__depthStep
        self.setFaceDepth(charIndex, depth)

    def setCharacterTile(self, charIndex: int, tileX: float, tileY: float):
        self.setFaceTileTexture(charIndex, tileX, tileY)

    def hideCharacter(self, charIndex: int):
        self.hideFace(charIndex)

    def getPixelCharacterIndices(self, x: int, y: int) -> List[int]:
        depth = int(round(self._gui.getPixelDepth(x, y)))
        if not self.hasDepth(depth):
            return []
        viewX, viewY = self._layerGroup.getTranslation()
        return self.findFaces(x + viewX, y + viewY)
