from __future__ import annotations

from typing import Dict

from gui.impl.opengl import OpenGLGUIFacade
from gui.layer import UILayer
from .OpenGLLayer import OpenGLLayer
from .OpenGLLayerGroup import OpenGLLayerGroup
from ..private import FrameBox


class OpenGLUILayer(OpenGLLayer, UILayer):

    def __init__(self, gui: OpenGLGUIFacade, layerGroup: OpenGLLayerGroup):
        super(OpenGLUILayer, self).__init__(gui, layerGroup, dynamic=True)
        self.__lastId = 0
        self.__frameBoxes = {}  # type: Dict[int, FrameBox]
        self.__depth = 0  # type: int

    def updateDepths(self, minDepth: int, maxDepth: int, index: int, maxIndex: int) -> int:
        self.__depth = minDepth
        self.setFacesDepth(self.__depth)
        return self.__depth - 1

    def hasDepth(self, depth: int) -> bool:
        return depth == self.__depth

    def createFrameBox(self, x1: float, y1: float, x2: float, y2: float) -> int:
        # Create id for new frame box
        self.__lastId += 1
        frameId = self.__lastId

        # Allocate faces
        facesIndices = self.allocateFaces(frameId, 9)
        frame = FrameBox(self, facesIndices, x1, y1, x2, y2, self.tileWidth, self.tileHeight)
        self.__frameBoxes[frameId] = frame

        # Defines the faces
        frame.show()

        return frameId

    def showFrameBox(self, frameId: int):
        self.__frameBoxes[frameId].show()

    def hideFrameBox(self, frameId: int):
        self.__frameBoxes[frameId].hide()

    def deleteFrameBox(self, frameId: int):
        # Free faces used by the frame box
        self.freeFaces(frameId)

        # Remove the frame box from the dict
        del self.__frameBoxes[frameId]




