import time
from typing import Union

import pygame

from gui.input import Mouse, Event
from .PygameMouseEvent import PygameMouseEvent


class PygameMouse(Mouse):

    def isButtonPressed(self, button: int) -> bool:
        assert 0 <= button < 3
        return pygame.mouse.get_pressed()[button]

    @property
    def x(self) -> int:
        return pygame.mouse.get_pos()[0]

    @property
    def y(self) -> int:
        return pygame.mouse.get_pos()[1]

    def createEvent(self, event) -> Union[None, PygameMouseEvent]:
        if event.type == pygame.MOUSEBUTTONDOWN:
            return PygameMouseEvent(Event.MOUSEBUTTONDOWN, event.button, event.pos[0], event.pos[1], time.time())
        elif event.type == pygame.MOUSEBUTTONUP:
            return PygameMouseEvent(Event.MOUSEBUTTONUP, event.button, event.pos[0], event.pos[1], time.time())
        return None
