from __future__ import annotations

from logic import Logic
from logic.command.Command import Command, CHARACTER_MOVE_PRIORITY
from state import State, Direction


class MoveCharacterCommand(Command):

    def __init__(self, state: State, regionName: str, characterIndex: int):
        self.__state = state
        self.__regionName = regionName
        self.__characterIndex = characterIndex

    def priority(self) -> int:
        return CHARACTER_MOVE_PRIORITY + self.__characterIndex

    def execute(self, logic: Logic):
        if self.__regionName is None:
            return
        region = self.__state.findRegion(self.__regionName)
        if region is None:
            return
        character = region.findCharacter(self.__characterIndex)
        if character is None:
            return

        moveDirection = character.moveDirection
        commandDirection = character.commandDirection
        if moveDirection == Direction.NONE and commandDirection == Direction.NONE:
            return

        shift = character.shift
        if shift >= character.speed:
            shift = 0
            moveDirection = Direction.NONE
            if commandDirection == Direction.NONE:
                character.moveDirection = Direction.NONE
                self.__state.notifyCharacterChanged(self.__state, self.__regionName, self.__characterIndex)
                return

        x = character.x
        y = character.y
        displayDirection = character.displayDirection
        if shift == 0 or commandDirection.isOppositeOf(moveDirection):
            displayDirection = commandDirection
            xp = x
            yp = y
            if commandDirection == Direction.NORTH:
                yp -= 1
            elif commandDirection == Direction.SOUTH:
                yp += 1
            elif commandDirection == Direction.WEST:
                xp -= 1
            elif commandDirection == Direction.EAST:
                xp += 1
            if region.canWalk(xp, yp):
                x = xp
                y = yp
                if commandDirection.isOppositeOf(moveDirection):
                    shift = character.speed - shift
                moveDirection = commandDirection
            else:
                moveDirection = Direction.NONE

        if moveDirection != Direction.NONE:
            shift += 1

        character.x = x
        character.y = y
        character.shift = shift
        character.displayDirection = displayDirection
        character.moveDirection = moveDirection
        character.commandDirection = Direction.NONE

        self.__state.notifyCharacterChanged(self.__state, self.__regionName, self.__characterIndex)



