from __future__ import annotations

from logic import Logic
from state import State
from logic.command.Command import Command, REGION_PRIORITY
from state.Region import REGION_MAX_WIDTH


class SetTreeValueCommand(Command):

    def __init__(self, state: State, regionName: str, x: int, y: int, value: int):
        self.__state = state
        self.__regionName = regionName
        self.__x = x
        self.__y = y
        self.__value = value

    def priority(self) -> int:
        return REGION_PRIORITY + self.__x + self.__y * REGION_MAX_WIDTH

    def execute(self, logic: Logic):
        region = self.__state.getRegion(self.__regionName)
        if not (0 <= self.__x < region.width and 0 <= self.__y < region.height):
            return
        if region.getTreeValue(self.__x, self.__y) == self.__value:
            return

        # Set the cell and notify
        region.setTreeValue(self.__x, self.__y, self.__value)
        self.__state.notifyRegionCellChanged(self.__regionName, self.__x, self.__y)
