import typing

from gui import GUIFacade
from gui.input import Keyboard, Event, MouseEvent, KeyEvent
from gui.layer import CharactersLayer, GridLayer
from gui.text import TextStyle
from logic.command import CharacterDirectionCommand, LoadStateCommand
from state import Direction
from state.Region import GROUND_ARRAY_INDEX, WATER_ARRAY_INDEX, TREE_ARRAY_INDEX, TREE_HERB_2, TREE_HERB_3, TREE_HERB_4, TREE_FLOWER_2, TREE_FLOWER_3, \
    TREE_FLOWER_4, TREE_MUSHROOM_2, TREE_NENUPHAR_2
from .DefaultGameMode import DefaultGameMode
from state.Region import GROUND_GRASS, GROUND_SWAMP, GROUND_DIRT, GROUND_SAND, GROUND_DEAD_LAND, WATER_DARK, WATER_RIVER, WATER_SWAMP, WATER_CAVE, \
    WATER_SEA, TREE_LIGHT_GREEN, TREE_GREEN, TREE_ORANGE, TREE_DEAD, TREE_STUMP_LARGE, TREE_HERB_1, TREE_FLOWER_1, \
    TREE_MUSHROOM_1, TREE_NENUPHAR_1


class PlayGameMode(DefaultGameMode):

    def __init__(self, gui: GUIFacade):
        super(PlayGameMode, self).__init__(gui)

        self.__frontLayerGroup = gui.createLayerGroup()
        self.__framesLayer = self.__frontLayerGroup.createUILayer()
        self.__framesLayer.setTileset("assets/ui/ui.png")
        self.__framesLayer.setTileSize(self._tileSize, self._tileSize)
        self.__framesLayer.createFrameBox(
            self._tileSize,
            gui.screenHeight - self._tileSize * 2,
            gui.screenWidth - self._tileSize,
            gui.screenHeight,
        )

        style = TextStyle("assets/font/basis33/basis33.ttf", self._tileSize // 2, (255, 255, 255))
        self.__textLayer = self.__frontLayerGroup.createTextLayer()
        self.__textLayer.setStyle(style)
        textBoxId = self.__textLayer.createTextBox(
            self._tileSize * 1.5,
            gui.screenHeight - self._tileSize * 1.75
        )
        self.__textLayer.setTextBoxContent(
            textBoxId,
            "Press a key to start an earthquake!\n"
            "1: random  2: random with fade  3: sinus  4: sinus with fade\n"
            "More details https://www.patternsgameprog.com"
        )

    #############################################################
    # Mode
    #############################################################

    def init(self):
        self._logic.addCommand(LoadStateCommand(self, 'level.bin'))

    def handleInputs(self) -> bool:

        # Keyboard
        keyboard = self._gui.keyboard
        direction = Direction.NONE
        if keyboard.isKeyPressed(Keyboard.K_LEFT):
            direction = Direction.WEST
        elif keyboard.isKeyPressed(Keyboard.K_RIGHT):
            direction = Direction.EAST
        elif keyboard.isKeyPressed(Keyboard.K_UP):
            direction = Direction.NORTH
        elif keyboard.isKeyPressed(Keyboard.K_DOWN):
            direction = Direction.SOUTH
        if direction != Direction.NONE:
            self._logic.addCommand(CharacterDirectionCommand(self._state, self._currentRegionName, 0, direction))
            return True

        # Mouse
        for event in self._gui.getEvents():
            if event.type == Event.KEYDOWN:
                event = typing.cast(KeyEvent, event)
                if event.key == Keyboard.K_1:
                    self._worldRenderer.startEarthquake(
                        "random", "",
                        self._gui.getTime(), self._gui.getTime() + 1000
                    )
                elif event.key == Keyboard.K_2:
                    self._worldRenderer.startEarthquake(
                        "random", "linear",
                        self._gui.getTime(), self._gui.getTime() + 1000
                    )
                elif event.key == Keyboard.K_3:
                    self._worldRenderer.startEarthquake(
                        "sinus", "",
                        self._gui.getTime(), self._gui.getTime() + 1000
                    )
                elif event.key == Keyboard.K_4:
                    self._worldRenderer.startEarthquake(
                        "sinus", "linear",
                        self._gui.getTime(), self._gui.getTime() + 1000
                    )
        return False

    def render(self, time: float, epochFrame: float):
        self._worldRenderer.updateAnimations(time, epochFrame)
        self._worldRenderer.updateViewOnCharacter(0)
        self._gui.render()

