from gui.layer import CharactersLayer, TextLayer
from state import Character, Direction
from .Animation import Animation


class CharacterAnimation(Animation):

    def __init__(self, layer: CharactersLayer, index: int, textLayer: TextLayer):
        self.__layer = layer
        self.__index = index

        self.__x = 0
        self.__y = 0
        self.__shift = 0
        self.__speed = 0
        self.__category = 0
        self.__moveDirection = Direction.NONE
        self.__displayDirection = Direction.NONE
        self.__speech = ""
        self.__speechEndTime = 0

        self.__textLayer = textLayer
        self.__textBoxId = 0

        self.__locationX = 0
        self.__locationY = 0

    def delete(self):
        if self.__textBoxId != 0:
            self.__textLayer.deleteTextBox(self.__textBoxId)
            self.__textBoxId = 0

    def getLocation(self) -> (float, float):
        return self.__locationX, self.__locationY

    def updateCharacter(self, character: Character, time: float):
        self.__x = character.x
        self.__y = character.y
        self.__shift = character.shift
        self.__speed = character.speed
        self.__category = character.category
        self.__moveDirection = character.moveDirection
        self.__displayDirection = character.displayDirection

        speech = character.speech
        speechEndTime = character.speechEndTime
        if speech != "" and speechEndTime > time:
            if self.__textBoxId == 0:
                self.__textBoxId = self.__textLayer.createTextBox()
            if speech != self.__speech or self.__speechEndTime != speechEndTime:
                self.__textLayer.setTextBoxContent(self.__textBoxId, speech)
                self.__speech = speech
                self.__speechEndTime = speechEndTime
        elif self.__textBoxId != 0:
            self.__textLayer.deleteTextBox(self.__textBoxId)
            self.__textBoxId = 0

    def update(self, time: float, epochFrame: float) -> bool:
        x = self.__x
        y = self.__y
        shift = self.__shift
        if shift > 0:
            shift = (shift - 1 + epochFrame) / self.__speed - 1.0
        direction = self.__moveDirection
        tileStep = round(3 * shift) % 3

        if direction == Direction.NORTH:
            y -= shift
        elif direction == Direction.SOUTH:
            y += shift
        elif direction == Direction.WEST:
            x -= shift
        elif direction == Direction.EAST:
            x += shift
        else:
            direction = self.__displayDirection
            tileStep = 1

        self.__locationX = x
        self.__locationY = y
        self.__layer.setCharacterLocation(self.__index, x, y)

        if direction == Direction.NORTH:
            tileX = 9
        elif direction == Direction.WEST:
            tileX = 3
        elif direction == Direction.EAST:
            tileX = 6
        else:
            tileX = 0
        tileY = self.__category - 1

        self.__layer.setCharacterTile(self.__index, tileX + tileStep, tileY)

        # Text
        if self.__textBoxId > 0:
            if time < self.__speechEndTime:
                textWidth, textHeight = self.__textLayer.getTextBoxSize(self.__textBoxId)
                textX = (x + 0.5) * self.__layer.tileWidth - textWidth // 2
                textY = y * self.__layer.tileHeight - textHeight
                self.__textLayer.setTextBoxLocation(self.__textBoxId, textX, textY)
            else:
                self.__textLayer.deleteTextBox(self.__textBoxId)
                self.__textBoxId = 0

        return True

