from __future__ import annotations

import io
from typing import Union

from state import State
from .IO import IO
from .v3.CharacterSerializer import CharacterSerializer
from .v3.DirectionSerializer import DirectionSerializer


class MainIO(IO):
    
    @staticmethod
    def createV1(stream: Union[None, io.BytesIO] = None) -> MainIO:
        from .v1 import StateSerializer
        from .v1 import RegionSerializer

        mainIO = MainIO(stream)
        mainIO.registerObjectSerializer("State", StateSerializer())
        mainIO.registerObjectSerializer("Region", RegionSerializer())

        return mainIO

    @staticmethod
    def createV2(stream: Union[None, io.BytesIO] = None) -> MainIO:
        from .v1 import StateSerializer
        from .v2 import RegionSerializer

        mainIO = MainIO(stream)
        mainIO.registerObjectSerializer("State", StateSerializer())
        mainIO.registerObjectSerializer("Region", RegionSerializer())

        return mainIO

    @staticmethod
    def createV3(stream: Union[None, io.BytesIO] = None) -> MainIO:
        from .v1 import StateSerializer
        from .v3 import RegionSerializer

        mainIO = MainIO(stream)
        mainIO.registerObjectSerializer("State", StateSerializer())
        mainIO.registerObjectSerializer("Region", RegionSerializer())
        mainIO.registerObjectSerializer("Character", CharacterSerializer())
        mainIO.registerObjectSerializer("Direction", DirectionSerializer())

        return mainIO

    @staticmethod
    def save(fileName: str, state: State):
        mainIO = MainIO.createV3()
        mainIO.createStream()
        mainIO.writeFileHeader("level", 3)
        mainIO.writeObject(state)
        mainIO.saveFile(fileName)

    @staticmethod
    def load(fileName: str) -> State:
        mainIO = IO()
        mainIO.loadFile(fileName)
        version = mainIO.readFileHeader("level")
        if version == 1:
            mainIO = MainIO.createV1(mainIO.stream)
            state = mainIO.readObject()
        elif version == 2:
            mainIO = MainIO.createV2(mainIO.stream)
            state = mainIO.readObject()
        elif version == 3:
            mainIO = MainIO.createV3(mainIO.stream)
            state = mainIO.readObject()
        else:
            raise RuntimeError("Unsupported version {}".format(version))
        return state
